# -*- coding: utf-8 -*-

import logging

from odoo import models, fields, _, api
from odoo.exceptions import ValidationError
from datetime import datetime
from datetime import date
from dateutil.relativedelta import relativedelta
from io import StringIO
import calendar
import base64


_logger = logging.getLogger(__name__)

class MedicalInvoice(models.Model):
    _name = 'medical.invoice'
    _description = 'Medical Invoice'

    def _compute_medical_benefit_ids_qty(self):
        self.medical_benefit_ids_qty = self.medical_benefit_ids.__len__()

    def _compute_account_invoice_ids_qty(self):
        self.account_invoice_ids_qty = self.account_invoice_ids.__len__()

    closing_day = fields.Integer(string='Closing day', required=True)
    start_date = fields.Date(string='Start date', required=True)
    end_date = fields.Date(string='End date', required=True)
    medical_insurance = fields.Many2many(comodel_name='res.partner', string='Medical insurance')
    medical_benefit_ids = fields.One2many('medical.benefit', 'medical_invoice_id', string="Medical benefits")
    account_invoice_ids = fields.One2many('account.invoice', 'medical_invoice_id', string="Medical invoice")
    reinvoice_benefits = fields.Boolean('Reinvoice benefits')
    medical_benefit_ids_qty = fields.Integer(string="Benefits", compute='_compute_medical_benefit_ids_qty')
    account_invoice_ids_qty = fields.Integer(string="Invoices", compute='_compute_account_invoice_ids_qty')
    state = fields.Selection([
        ('draft', 'Draft'),
        ('in_progress', 'In Progress'),
        ('draft_invoice', 'Draft Invoice'),
        ('invoiced', 'Invoiced')
    ], string='State', default='draft')

    _sql_constraints = [(
        'check_date',
        'CHECK(end_date >= start_date)',
        _('The end date must be greater than the start date')
    )]

    @api.onchange('start_date')
    def onchange_start_date(self):
        sd = self.start_date
        if sd:
            last_day = calendar.mdays[sd.month]
            last_day += 1 if calendar.isleap(sd.year) and sd.month == 2 else 0
            self.end_date = date(sd.year,sd.month,last_day)


    # ----- Buttons ----- #

    @api.constrains('start_date', 'end_date')
    def _validate_same_month_perido(self):
        if (self.start_date.year != self.end_date.year or
            self.start_date.month != self.end_date.month):
            raise ValidationError(
                _('Start date and end date must belong to the same month')
            )

    # ----- Buttons ----- #

    @api.multi
    def action_search(self):
        hip_obj = self.env['health.insurance.plan']
        mb_obj = self.env['medical.benefit']
        medical_benefit_list = list()
        for partner in self.medical_insurance:
            medical_benefit_list += mb_obj.search([
                ('insurance_plan_id', '=', partner.insurance_plan_id.id),
                ('presentation_date', '>=', self.start_date),
                ('presentation_date', '<=', self.end_date),
                ('state', '=', 'for_invoicing' if not self.reinvoice_benefits else 'for_reinvoicing')
            ]).ids
        self.medical_benefit_ids = medical_benefit_list
        if self.state == 'draft':
            self.state = 'in_progress'

    @api.multi
    def action_charge_benefits(self):
        context = dict()
        context.update(self.env.context)
        context.update({'create': False, 'group_by': ['insurance_plan_id', 'patient_id']})
        action = self.env.ref('medical_benefit.medical_benefit_action').read()[0]
        action['domain'] = str([('id', 'in', self.medical_benefit_ids.ids)])
        action['context'] = context
        return action

    @api.multi
    def action_charge_invoices(self):
        context = dict()
        context.update(self.env.context)
        context.update({'create': False})
        action = self.env.ref('account.action_invoice_tree1').read()[0]
        action['domain'] = str([('id', 'in', self.account_invoice_ids.ids)])
        action['context'] = context
        return action

    @api.multi
    def action_to_draft(self):
        self.medical_benefit_ids = False
        self.state = 'draft'

    @api.multi
    def action_in_progress(self):
        self.delete_existing_invoices()
        self.state = 'in_progress'

    @api.multi
    def action_draft_invoice(self):
        self.delete_existing_invoices()
        AI_obj = self.env['account.invoice']
        res_partner = self.medical_insurance
        date_invoice = str(datetime.now())
        for partner in res_partner:
            insurance_config = self.env['medical.invoice.config'].search([('insurance_id', '=', partner.insurance_plan_id.insurance_id.id)])
            date_due_invoice = str(datetime.now() + relativedelta(
                days=insurance_config.due_days if insurance_config else 30)
            )
            if insurance_config.is_capitated and not insurance_config.pre_invoice_medical_action:
                description_date = ' ' + '-'.join(str(self.start_date).split('-')[0:2][::-1])
                invoice_line = [{
                    'name': _('Month') + description_date,
                    'account_id': insurance_config.product_id.categ_id.property_account_income_categ_id.id,
                    'product_id': insurance_config.product_id.id,
                    'price_unit': insurance_config.capitated_payment_amount,
                    'quantity': 1,
                }]
                invoice_line_ids = self.env['account.invoice.line'].create(invoice_line).ids
            else:
                benefits = self.medical_benefit_ids.filtered(
                    lambda r, partner=partner: r.insurance_plan_id == partner.insurance_plan_id
                )
                if not benefits:
                    continue
                account_id = insurance_config.product_id.categ_id.property_account_income_categ_id.id
                product_id = insurance_config.product_id.id
                invoice_lines = [{
                    'name': rec.name_get()[0][1],
                    'price_unit': rec.price / rec.quantity,
                    'quantity': rec.quantity,
                    'account_id': account_id,
                    'product_id': product_id,
                    'benefit_id': rec.id,
                    'descriptor_id': rec.descriptor_id.id,
                    'professional_id': rec.professional_id.id,
                } for rec in benefits ]
                invoice_line_ids = self.env['account.invoice.line'].create(invoice_lines).ids
            invoice = AI_obj.create({
                'currency_id': self.env.user.company_id.currency_id.id,
                'date_invoice': date_invoice,
                'partner_id': partner.id,
                'journal_id': 1,
                'company_id': self.env.user.company_id.id,
                'name': partner.name,
                'type': 'out_invoice',
                'medical_invoice_id': self.id,
                'invoice_line_ids': [[6, 0] + [invoice_line_ids]]
            })
            invoice._onchange_partner_id()
            invoice.write({'date_due': date_due_invoice})
            for invoice_line in invoice.invoice_line_ids:
                company_id = invoice_line.company_id or self.env.user.company_id
                taxes = invoice_line.product_id.taxes_id.filtered(
                    lambda r: r.company_id == company_id
                ) or invoice_line.account_id.tax_ids or invoice_line.invoice_id.company_id.account_sale_tax_id
                if invoice_line.invoice_id.partner_id.property_account_position_id.id != self.env.ref('l10n_ar_point_of_sale.fiscal_position_exempt').id:
                    invoice_line.invoice_line_tax_ids = invoice_line.professional_id.property_account_position_id.map_tax(
                        taxes, invoice_line.product_id, invoice_line.professional_id
                    )
            invoice._onchange_invoice_line_ids()
        self.state = 'draft_invoice'

    @api.multi
    def action_invoiced(self):
        for invoice in self.account_invoice_ids:
            invoice.action_invoice_open()
        self.state = 'invoiced'

    # ----- Search medical insurance ----- #

    @api.onchange('closing_day')
    def on_change_closing_day(self):
        for rec in self:
            recs = self.env['medical.invoice.config'].search([
                ('filing_day', '=', rec.closing_day)
            ])
            insurance_ids = []
            for config in recs:
                insurance_ids.extend(config.insurance_id.health_insurance_plan_ids.mapped('partner_id').ids)
            #rec.medical_insurance = [(6, 0, [e.insurance_id.health_insurance_plan_ids.partner_id for e in recs])]
            rec.medical_insurance = [(6, 0, insurance_ids)]

    @api.multi
    def name_get(self):
        def fstr(rec):
            return _('Close day') + f' {str(rec.closing_day)} ' + _('from') \
                + f' {str(rec.start_date)} ' + _('to') + f' {str(rec.end_date)}'
        return [(r.id, fstr(r)) for r in self]

    # ----------------------------------- #

    @api.multi
    def delete_existing_invoices(self):
        for invoice in self.account_invoice_ids:
            invoice.unlink()


    @api.multi
    @api.returns('ir.attachment')
    def create_txt_UNL(self):
        string_file = StringIO()
        string_file.write('This is a test file!!!!')
        file_data = string_file.getvalue()
        values = {
            'name': 'TextTest.txt',
            'datas_fname': 'print_text.txt',
            'res_model': 'ir.ui.view',
            'res_id': False,
            'type': 'binary',
            'public': True,
            #'datas': file_data.encode('utf8'),
            'datas': base64.b64encode(file_data.encode('utf8')),
        }
        attachment_id = self.env['ir.attachment'].sudo().create(values)
        download_url = '/web/content/' + str(attachment_id.id) + '?download=True'
        base_url = self.env['ir.config_parameter'].get_param('web.base_url')
        return {
                'type': 'ir.actions.act_url',
                'url': str(base_url) + str(download_url),
                'target': 'new',
        }

