# -*- coding: utf-8 -*-
###############################################################################
#   Copyright (c) 2021 Eynes/E-mips (Marco Folco)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
###############################################################################

import logging

from odoo import models, api, fields, _
from odoo.exceptions import ValidationError, UserError
from datetime import datetime
from dateutil.relativedelta import relativedelta
# from odoo.exceptions import except_orm
# from odoo.addons.decimal_precision import decimal_precision as dp
# from odoo.tools import DEFAULT_SERVER_DATE_FORMAT, \
#         DEFAULT_SERVER_DATETIME_FORMAT, float_compare


_logger = logging.getLogger(__name__)


class WizardAccountInvoiceModify(models.TransientModel):
    _name = "wizard.account.invoice.modify"
    _description = "Wizard to modify lines of an invoice"

    state = fields.Selection([
        ('change_tariff', 'Change Tariff'),
        ('change_insurance', 'Change Insurance')
    ], string='Action', default='change_tariff')

    descriptor_id = fields.Many2one('medical.benefit.descriptor','Descriptor',domain=lambda x: x.get_domain_descriptor_id())
    professional_id = fields.Many2one('res.partner','Professional',domain=lambda x: x.get_domain_professional_id())
    date_from = fields.Date('Date from')
    date_to = fields.Date('Date to')
    price = fields.Float('Price')
    insurance_id = fields.Many2one('health.insurance.plan','New Insurance',domain=lambda x: x.get_domain_insurance_id())

    @api.multi
    def get_domain_insurance_id(self):
        domain = []
        invoice = self.env['account.invoice'].browse(self._context.get('active_id',False))
        plan = invoice.partner_id.insurance_plan_id
        if plan:
            domain.extend([('insurance_id','=',plan.insurance_id.id),('id','!=',plan.id)])
        return domain

    @api.multi
    def get_domain_professional_id(self):
        domain = []
        invoice = self.env['account.invoice'].browse(self._context.get('active_id',False))
        professionals = invoice.invoice_line_ids.mapped('professional_id')
        domain.append(('id','in',professionals.ids if professionals else []))
        return domain

    @api.multi
    def get_domain_descriptor_id(self):
        domain = []
        invoice = self.env['account.invoice'].browse(self._context.get('active_id',False))
        descriptors = invoice.invoice_line_ids.mapped('descriptor_id')
        domain.append(('id','in',descriptors.ids if descriptors else []))
        return domain

    @api.onchange('descriptor_id')
    def onchange_descriptor_id(self):
        invoice = self.env['account.invoice'].browse(self._context.get('active_id',False))
        if self.descriptor_id:
            lines = invoice.invoice_line_ids.filtered(lambda x:x.descriptor_id.id == self.descriptor_id.id)
            if lines:
                self.price = lines[0].price_unit
            else:
                self.price = 0
        else:
            self.price = 0

    @api.multi
    def filter_lines(self):        
        invoice = self.env['account.invoice'].browse(self._context.get('active_id',False))
        invoice_lines = invoice.invoice_line_ids
        if self.descriptor_id:
            invoice_lines = invoice_lines.filtered(lambda x:x.descriptor_id == self.descriptor_id)
        if self.professional_id:
            invoice_lines = invoice_lines.filtered(lambda x:x.professional_id == self.professional_id)
        if self.date_from:
            invoice_lines = invoice_lines.filtered(lambda x:x.benefit_date >= self.date_from)
        if self.date_to:
            invoice_lines = invoice_lines.filtered(lambda x:x.benefit_date <= self.date_to)
        return invoice_lines

    @api.multi
    def button_change_tariff(self):
        invoice_lines = self.filter_lines()
        if invoice_lines:
            invoice_lines.write({
                'price_unit': self.price,
            })
    
    @api.multi
    def button_change_insurance(self):
        current_invoice_lines = self.filter_lines()
        current_invoice = self.env['account.invoice'].browse(self._context.get('active_id',False))
        AI_obj = self.env['account.invoice']
        insurance_plan = self.insurance_id
        date_invoice = str(datetime.now())
        for plan in insurance_plan:
            insurance_config = self.env['medical.invoice.config'].search([('insurance_id', '=', plan.insurance_id.id)])
            date_due_invoice = str(datetime.now() + relativedelta(
                days=insurance_config.due_days if insurance_config else 1)
            )
            benefits = current_invoice_lines.mapped('benefit_id')
            if not benefits:
                continue
            account_id = insurance_config.product_id.categ_id.property_account_income_categ_id.id
            product_id = insurance_config.product_id.id
            invoice_lines = [{
                'name': rec.name_get()[0][1],
                'price_unit': rec.price / rec.quantity,
                'quantity': rec.quantity,
                'account_id': account_id,
                'product_id': product_id,
                'benefit_id': rec.id,
                'descriptor_id': rec.descriptor_id.id,
                'professional_id': rec.professional_id.id,
            } for rec in benefits ]
            invoice_line_ids = self.env['account.invoice.line'].create(invoice_lines).ids
            invoice = AI_obj.create({
                'currency_id': self.env.user.company_id.currency_id.id,
                'date_invoice': date_invoice,
                'partner_id': plan.partner_id.id,
                'journal_id': 1,
                'company_id': self.env.user.company_id.id,
                'name': plan.partner_id.name,
                'type': 'out_invoice',
                'medical_invoice_id': current_invoice.medical_invoice_id.id,
                'invoice_line_ids': [[6, 0] + [invoice_line_ids]]
            })
            invoice._onchange_partner_id()
            invoice.write({'date_due': date_due_invoice})
            for invoice_line in invoice.invoice_line_ids:
                company_id = invoice_line.company_id or self.env.user.company_id
                taxes = invoice_line.product_id.taxes_id.filtered(
                    lambda r: r.company_id == company_id
                ) or invoice_line.account_id.tax_ids or invoice_line.invoice_id.company_id.account_sale_tax_id
                invoice_line.invoice_line_tax_ids = invoice_line.professional_id.property_account_position_id.map_tax(
                    taxes, invoice_line.product_id, invoice_line.professional_id
                )
            invoice._onchange_invoice_line_ids()
        for line in current_invoice_lines:
            current_invoice.write({'invoice_line_ids': [(2,line.id)]})
        if not current_invoice.invoice_line_ids:
            current_invoice.unlink()

        return {
            'context': self.env.context,
            'view_type': 'form',
            'view_mode': 'form',
            'res_model': 'account.invoice',
            'res_id': invoice.id,
            'view_id': self.env.ref('account.invoice_form').id,
            'type': 'ir.actions.act_window',
        }


